@@ -357,6 +357,11 @@ gsl_nls.formula <- function(fn, data = parent.frame(), start,
 
   .lhs <- eval(formula[[2L]], envir = mf)
 
+  ## n >= p
+  if(length(.lhs) < length(start)) {
+    stop("negative residual degrees of freedom, cannot fit a model with less observations than parameters")
+  }
+
   ## jac call
   if(!is.function(jac) && !is.null(attr(.fcall, "gradient"))) {
     jac <- function(par, .data = mf) attr(.fn(par, .data), "gradient")
@@ -516,6 +521,11 @@ gsl_nls.function <- function(fn, y, start,
   if(!is.numeric(y))
     stop("'y' should be a numeric response vector")
 
+  ## n >= p
+  if(length(y) < length(start)) {
+    stop("negative residual degrees of freedom, cannot fit a model with less observations than parameters")
+  }
+
   .fn <- function(par) fn(par, ...)
   .fcall <- tryCatch(.fn(start), error = function(err) err)
   if(inherits(.fcall, "error"))
