@@ -2,24 +2,50 @@
 
 #include "gsl_nls.h"
 
+/* cleanup memory */
+static void C_nls_cleanup(void *data)
+{
+    pdata *pars = data;
+
+    /* free memory */
+    if (pars->w)
+        gsl_multifit_nlinear_free(pars->w);
+
+}
+
+/* function call w/ cleanup */
 SEXP C_nls(SEXP fn, SEXP y, SEXP jac, SEXP fvv, SEXP env, SEXP start, SEXP swts, SEXP control_int, SEXP control_dbl)
+{
+    /* function arguments */
+    pdata pars = {fn, y, jac, fvv, env, start, swts, control_int, control_dbl, NULL};
+
+    /* safe function call */
+    SEXP ans = R_ExecWithCleanup(C_nls_internal, &pars, C_nls_cleanup, &pars);
+
+    return ans;
+}
+
+SEXP C_nls_internal(void *data)
 {
     /* turn off error handler to avoid aborting on errors */
     gsl_set_error_handler_off();
 
+    /* function arguments */
+    pdata *pars = data;
+
     /* initialize parameters */
     int nprotect = 1;
-    SEXP startvec = PROTECT(Rf_coerceVector(start, REALSXP));
+    SEXP startvec = PROTECT(Rf_coerceVector(pars->start, REALSXP));
     R_len_t p = Rf_length(startvec);
-    R_len_t n = Rf_length(y);
-    R_len_t niter = INTEGER_ELT(control_int, 0);
-    int verbose = INTEGER_ELT(control_int, 1);
+    R_len_t n = Rf_length(pars->y);
+    R_len_t niter = INTEGER_ELT(pars->control_int, 0);
+    int verbose = INTEGER_ELT(pars->control_int, 1);
 
     /* control parameters */
     gsl_multifit_nlinear_parameters fdf_params = gsl_multifit_nlinear_default_parameters();
 
     /* minimization algorithm */
-    switch (INTEGER_ELT(control_int, 2))
+    switch (INTEGER_ELT(pars->control_int, 2))
     {
     case 1:
         fdf_params.trs = gsl_multifit_nlinear_trs_lmaccel;
@@ -38,7 +64,7 @@ SEXP C_nls(SEXP fn, SEXP y, SEXP jac, SEXP fvv, SEXP env, SEXP start, SEXP swts,
     }
 
     /* scaling method */
-    switch (INTEGER_ELT(control_int, 3))
+    switch (INTEGER_ELT(pars->control_int, 3))
     {
     case 1:
         fdf_params.scale = gsl_multifit_nlinear_scale_levenberg;
@@ -51,7 +77,7 @@ SEXP C_nls(SEXP fn, SEXP y, SEXP jac, SEXP fvv, SEXP env, SEXP start, SEXP swts,
     }
 
     /* solver type */
-    switch (INTEGER_ELT(control_int, 4))
+    switch (INTEGER_ELT(pars->control_int, 4))
     {
     case 1:
         fdf_params.solver = gsl_multifit_nlinear_solver_cholesky;
@@ -64,22 +90,22 @@ SEXP C_nls(SEXP fn, SEXP y, SEXP jac, SEXP fvv, SEXP env, SEXP start, SEXP swts,
     }
 
     /* finite differencing type */
-    fdf_params.fdtype = INTEGER_ELT(control_int, 5) ? GSL_MULTIFIT_NLINEAR_CTRDIFF : GSL_MULTIFIT_NLINEAR_FWDIFF;
+    fdf_params.fdtype = INTEGER_ELT(pars->control_int, 5) ? GSL_MULTIFIT_NLINEAR_CTRDIFF : GSL_MULTIFIT_NLINEAR_FWDIFF;
 
     /* miscellaneous parameters */
-    fdf_params.factor_up = REAL_ELT(control_dbl, 0);
-    fdf_params.factor_down = REAL_ELT(control_dbl, 1);
-    fdf_params.avmax = REAL_ELT(control_dbl, 2);
-    fdf_params.h_df = REAL_ELT(control_dbl, 3);
-    fdf_params.h_fvv = REAL_ELT(control_dbl, 4);
-    double xtol = REAL_ELT(control_dbl, 5);
-    double ftol = REAL_ELT(control_dbl, 6);
-    double gtol = REAL_ELT(control_dbl, 7);
+    fdf_params.factor_up = REAL_ELT(pars->control_dbl, 0);
+    fdf_params.factor_down = REAL_ELT(pars->control_dbl, 1);
+    fdf_params.avmax = REAL_ELT(pars->control_dbl, 2);
+    fdf_params.h_df = REAL_ELT(pars->control_dbl, 3);
+    fdf_params.h_fvv = REAL_ELT(pars->control_dbl, 4);
+    double xtol = REAL_ELT(pars->control_dbl, 5);
+    double ftol = REAL_ELT(pars->control_dbl, 6);
+    double gtol = REAL_ELT(pars->control_dbl, 7);
 
     /* initialize data */
     SEXP xpar = Rf_install("par");
-    SEXP fcall = PROTECT(Rf_lang2(fn, xpar));
-    SEXP parnames = PROTECT(Rf_getAttrib(start, R_NamesSymbol));
+    SEXP fcall = PROTECT(Rf_lang2(pars->fn, xpar));
+    SEXP parnames = PROTECT(Rf_getAttrib(pars->start, R_NamesSymbol));
     nprotect += 2;
 
     fdata params;
@@ -88,9 +114,9 @@ SEXP C_nls(SEXP fn, SEXP y, SEXP jac, SEXP fvv, SEXP env, SEXP start, SEXP swts,
     params.f = fcall;
     params.df = NULL;
     params.fvv = NULL;
-    params.y = y;
-    params.rho = env;
-    params.start = start;
+    params.y = pars->y;
+    params.rho = pars->env;
+    params.start = pars->start;
 
     if (verbose)
     {
@@ -112,19 +138,19 @@ SEXP C_nls(SEXP fn, SEXP y, SEXP jac, SEXP fvv, SEXP env, SEXP start, SEXP swts,
     fdf.params = &params;
 
     /* use Jacobian function */
-    if (!Rf_isNull(jac))
+    if (!Rf_isNull(pars->jac))
     {
-        SEXP dfcall = PROTECT(Rf_lang2(jac, xpar));
+        SEXP dfcall = PROTECT(Rf_lang2(pars->jac, xpar));
         params.df = dfcall;
         fdf.df = gsl_df;
         nprotect++;
     }
 
     /* use acceleration function */
-    if (!Rf_isNull(fvv))
+    if (!Rf_isNull(pars->fvv))
     {
         SEXP vpar = Rf_install("v");
-        SEXP fvvcall = PROTECT(Rf_lang3(fvv, xpar, vpar));
+        SEXP fvvcall = PROTECT(Rf_lang3(pars->fvv, xpar, vpar));
         params.fvv = fvvcall;
         fdf.fvv = gsl_fvv;
         nprotect++;
@@ -138,28 +164,27 @@ SEXP C_nls(SEXP fn, SEXP y, SEXP jac, SEXP fvv, SEXP env, SEXP start, SEXP swts,
 
     /* initialize solver */
     const gsl_multifit_nlinear_type *T = gsl_multifit_nlinear_trust;
-    gsl_multifit_nlinear_workspace *w;
 
     /* allocate workspace with default parameters */
-    w = gsl_multifit_nlinear_alloc(T, &fdf_params, n, p);
+    pars->w = gsl_multifit_nlinear_alloc(T, &fdf_params, n, p);
 
     /* initialize solver with starting point and weights */
-    if (!Rf_isNull(swts))
+    if (!Rf_isNull(pars->swts))
     {
         double *swts1 = (double *)S_alloc(n, sizeof(double));
         for (R_len_t i = 0; i < n; i++)
-            swts1[i] = REAL_ELT(swts, i);
+            swts1[i] = REAL_ELT(pars->swts, i);
         gsl_vector_view wts = gsl_vector_view_array(swts1, n);
-        gsl_multifit_nlinear_winit(&par.vector, &wts.vector, &fdf, w);
+        gsl_multifit_nlinear_winit(&par.vector, &wts.vector, &fdf, pars->w);
     }
     else
     {
-        gsl_multifit_nlinear_init(&par.vector, &fdf, w);
+        gsl_multifit_nlinear_init(&par.vector, &fdf, pars->w);
     }
 
     /* compute initial cost function */
     double chisq_init = GSL_POSINF;
-    gsl_vector *resid = gsl_multifit_nlinear_residual(w);
+    gsl_vector *resid = gsl_multifit_nlinear_residual(pars->w);
     gsl_blas_ddot(resid, resid, &chisq_init);
     double chisq0 = chisq_init;
     double chisq1 = chisq_init;
@@ -175,8 +200,8 @@ SEXP C_nls(SEXP fn, SEXP y, SEXP jac, SEXP fvv, SEXP env, SEXP start, SEXP swts,
 
     /* solve the system  */
     int info = GSL_CONTINUE;
-    int status = gsl_multifit_nlinear_driver2(niter, xtol, gtol, ftol, verbose ? callback : NULL, verbose ? &params : NULL, &info, &chisq0, &chisq1, w);
-    R_len_t iter = gsl_multifit_nlinear_niter(w);
+    int status = gsl_multifit_nlinear_driver2(niter, xtol, gtol, ftol, verbose ? callback : NULL, verbose ? &params : NULL, &info, &chisq0, &chisq1, pars->w);
+    R_len_t iter = gsl_multifit_nlinear_niter(pars->w);
 
     /* compute covariance and cost at best fit parameters */
     gsl_matrix *J = NULL;
@@ -184,14 +209,14 @@ SEXP C_nls(SEXP fn, SEXP y, SEXP jac, SEXP fvv, SEXP env, SEXP start, SEXP swts,
     if (status == GSL_SUCCESS || status == GSL_EMAXITER)
     {
         cov = gsl_matrix_alloc(p, p);
-        J = gsl_multifit_nlinear_jac(w);
+        J = gsl_multifit_nlinear_jac(pars->w);
         gsl_multifit_nlinear_covar(J, 0.0, cov);
     }
 
     if (verbose)
     {
         /* print summary statistics*/
-        Rprintf("*******************\nsummary from method 'multifit/%s'\n", gsl_multifit_nlinear_trs_name(w));
+        Rprintf("*******************\nsummary from method 'multifit/%s'\n", gsl_multifit_nlinear_trs_name(pars->w));
         Rprintf("number of iterations: %d\n", iter);
         Rprintf("reason for stopping: %s\n", gsl_strerror(info));
         Rprintf("initial ssr = %g\n", chisq_init);
@@ -225,7 +250,7 @@ SEXP C_nls(SEXP fn, SEXP y, SEXP jac, SEXP fvv, SEXP env, SEXP start, SEXP swts,
     if (status == GSL_SUCCESS || status == GSL_EMAXITER)
     {
         for (R_len_t k = 0; k < p; k++)
-            SET_REAL_ELT(anspar, k, gsl_vector_get(w->x, k));
+            SET_REAL_ELT(anspar, k, gsl_vector_get(pars->w->x, k));
     }
     else
     {
@@ -317,7 +342,7 @@ SEXP C_nls(SEXP fn, SEXP y, SEXP jac, SEXP fvv, SEXP env, SEXP start, SEXP swts,
     SET_VECTOR_ELT(ans, 6, Rf_ScalarInteger(status));
     SET_VECTOR_ELT(ans, 7, Rf_ScalarReal(chisq1));
     SET_VECTOR_ELT(ans, 8, Rf_ScalarReal(chisq0 - chisq1));
-    SET_VECTOR_ELT(ans, 9, Rf_ScalarString(Rf_mkChar(gsl_multifit_nlinear_trs_name(w))));
+    SET_VECTOR_ELT(ans, 9, Rf_ScalarString(Rf_mkChar(gsl_multifit_nlinear_trs_name(pars->w))));
 
     const char *nms[] = {"f", "J", "fvv", ""};
     SEXP ansneval = PROTECT(Rf_mkNamed(INTSXP, nms));
@@ -342,11 +367,10 @@ SEXP C_nls(SEXP fn, SEXP y, SEXP jac, SEXP fvv, SEXP env, SEXP start, SEXP swts,
     }
 
     /* free memory */
-    UNPROTECT(nprotect);
-    gsl_multifit_nlinear_free(w);
     if (status == GSL_SUCCESS || status == GSL_EMAXITER)
         gsl_matrix_free(cov);
 
+    UNPROTECT(nprotect);
     return ans;
 }
 
